@@ -32,7 +32,7 @@ typedef  unsigned long  int  ub4;   /* unsigned 4-byte quantities */
 typedef  unsigned       char ub1;   /* unsigned 1-byte quantities */
 
 /* how many powers of 2's worth of buckets we use */
-static unsigned int hashpower = 16;
+static unsigned int hashpower = HASHPOWER_DEFAULT;
 
 #define hashsize(n) ((ub4)1<<(n))
 #define hashmask(n) (hashsize(n)-1)
@@ -58,12 +58,19 @@ static bool expanding = false;
  */
 static unsigned int expand_bucket = 0;
 
-void assoc_init(void) {
+void assoc_init(const int hashtable_init) {
+    if (hashtable_init) {
+        hashpower = hashtable_init;
+    }
     primary_hashtable = calloc(hashsize(hashpower), sizeof(void *));
     if (! primary_hashtable) {
         fprintf(stderr, "Failed to init hashtable.\n");
         exit(EXIT_FAILURE);
     }
+    STATS_LOCK();
+    stats.hash_power_level = hashpower;
+    stats.hash_bytes = hashsize(hashpower) * sizeof(void *);
+    STATS_UNLOCK();
 }
 
 item *assoc_find(const char *key, const size_t nkey) {
@@ -126,6 +133,11 @@ static void assoc_expand(void) {
         hashpower++;
         expanding = true;
         expand_bucket = 0;
+        STATS_LOCK();
+        stats.hash_power_level = hashpower;
+        stats.hash_bytes += hashsize(hashpower) * sizeof(void *);
+        stats.hash_is_expanding = 1;
+        STATS_UNLOCK();
         pthread_cond_signal(&maintenance_cond);
     } else {
         primary_hashtable = old_hashtable;
@@ -213,6 +225,10 @@ static void *assoc_maintenance_thread(void *arg) {
             if (expand_bucket == hashsize(hashpower - 1)) {
                 expanding = false;
                 free(old_hashtable);
+                STATS_LOCK();
+                stats.hash_bytes -= hashsize(hashpower - 1) * sizeof(void *);
+                stats.hash_is_expanding = 0;
+                STATS_UNLOCK();
                 if (settings.verbose > 1)
                     fprintf(stderr, "Hash table expansion done\n");
             }
